import filecmp
import json
import md5
import optparse
import os
import sys

# File Extensions
ExtensionMetadata   = '.cmmetadata'
ExtensionDescFile   = '.cmconfig'

# Metadata Fields
FieldSize           = 'size'
FieldChecksum       = 'checksum'
FieldEncrypted      = 'encrypted'

# Logging
LogVerboseEnabled   = False
LogErrorEnabled     = True


#-------------------------------------------------------------------------------
# Outputs a standard log. Always print to the output.
#
# \param log: string
#   The log to output.
#
#-------------------------------------------------------------------------------
def LogStd(log):
    print log
    print


#-------------------------------------------------------------------------------
# Outputs a verbose log if verbose logging is enabled.
#
# \param log: string
#   The log to output.
#
#-------------------------------------------------------------------------------
def LogVerbose(log):
    if LogVerboseEnabled:
        print log
        print


#-------------------------------------------------------------------------------
# Outputs an error log if error logging is enabled.
#
# \param log: string
#   The log to output.
#
#-------------------------------------------------------------------------------
def LogError(log):
    if LogErrorEnabled:
        print log
        print


#-------------------------------------------------------------------------------
# Tells if this type of file needs a Metadata, based on the file name and extension.
#
# \return: string
#   The MD5 for this fila
#
#-------------------------------------------------------------------------------
def ParseArgs():
    # Setup the command line parser.
    usage = "Usage: %prog [options]"

    parser = optparse.OptionParser(usage)

    parser.description = "Create Metadata file from a file containing any kind of files."

    parser.add_option("-v", "--verbose", action="store_true", dest="verbose", default=False,
        help="run in verbose mode.")

    parser.add_option("--root", action="store", dest="rootFolder", default="",
        help="set the root folder, where to list the files to add metadata to. Absolute or relative to the current woring directory.")

    parser.add_option("--size", action="store_true", dest="size", default=False,
        help="add the 'size' flag to all the Metadata files. The game is responsible for providing all the data required to decrypt the file at runtime.")

    parser.add_option("--md5All", action="store_true", dest="md5All", default=False,
        help="add the 'checksum' flag using md5 for all files, including the Metadata files.")

    parser.add_option("--md5ContentFile", action="store_true", dest="md5ContentFile", default=False,
        help="add the 'checksum' flag using md5 for all the content files, excluding the Metadata files.")

    parser.add_option("--md5MetadataFile", action="store_true", dest="md5MetadataFile", default=False,
        help="add the 'checksum' flag using md5 for all Metadata files only.")

    parser.add_option("--encrypted", action="store_true", dest="encrypted", default=False,
        help="add the 'encrypted' flag to all the Metadata files. The game is responsible for providing all the data required to decrypt the file at runtime.")

    # Parse the command line.
    return parser.parse_args()


#-------------------------------------------------------------------------------
# Tells if this type of file needs a Metadata, based on the file name and extension.
#
# \param filePath: string
#   The path of the file to hash and get the MD5.
#
# \return: boolean
#   True if this file should have a Metadata, False otherwise.
#
#-------------------------------------------------------------------------------
def ShouldCreateMetadata(filePath):
    pathExtension = os.path.splitext(filePath)[1]
    LogVerbose('File %s, Extension: %s' % (filePath, pathExtension))

    shouldCreate = True
    shouldCreate &= filePath != __file__
    shouldCreate &= pathExtension != ExtensionMetadata
    shouldCreate &= pathExtension != ExtensionDescFile

    LogVerbose('Should Create the Metadata: %s' % (str(shouldCreate)))

    return  shouldCreate


#-------------------------------------------------------------------------------
# Tells if a file contains exactly the given string.
#
# \param filePath: string
#   The path of the file to compare.
#
# \param expectedContent: string
#   The content to compare with.
#
# \return: boolean
#   True if the file is the same as the string, False otherwise.
#
#-------------------------------------------------------------------------------
def FileContentIs(filePath, expectedContent):
    f = open(filePath, "r")
    fileContent = f.read()
    f.close()

    return fileContent == expectedContent


#-------------------------------------------------------------------------------
# Tells if a file contains exactly the given string.
#
# \param filePath: string
#   The path of the file to write.
#
# \param fileContent: string
#   The content to write.
#
#-------------------------------------------------------------------------------
def WriteFileContent(filePath, fileContent):
    try:
        f = open(filePath, "w")
        f.write(fileContent)
        f.close()
    except:
        LogError('Error writing to %s. Is the file read-only?' % (filePath))


#-------------------------------------------------------------------------------
# Appends a file with the MD5 of the current content.
#
# \param filePath: string
#   The path of the file to append.
#
#-------------------------------------------------------------------------------
def AppendMD5(filePath):
    md5 = GetMD5(filePath)
    try:
        f = open(filePath, "a")
        f.write(md5)
        f.close()
    except:
        LogError('Error appending MD5 to %s. Is the file read-only?' % (filePath))


#-------------------------------------------------------------------------------
# Generates a MD5 from a file.
#
# \param filePath: string
#   The path of the file to hash and get the MD5.
#
# \return: string
#   The MD5 for this file.
#
#-------------------------------------------------------------------------------
def GetMD5(filePath):
    BlockSize = 1024*1024

    checksum = md5.new()
    f = open(filePath, "rb")
    while 1:
        block = f.read(BlockSize)
        if not block:
            break
        checksum.update(block)
    f.close()

    return checksum.hexdigest()


#-------------------------------------------------------------------------------
# Creates a single Metadata File.
#
# \param filePath: string
#   The path of the file that the new Metadata File should desbribe.
#
# \param options: object
#   The list of command lines options, parsed to control the Metadata fields.
#
#-------------------------------------------------------------------------------
def CreateMetadata(filePath, options):
    LogVerbose('Creating metadata for file %s' % (filePath))

    # Store the content of the metadata file in a dictionary where the key is the field
    metadataContent = {}

    if options.size:
        metadataContent[FieldSize]      = os.path.getsize(filePath)

    if options.md5ContentFile:
        metadataContent[FieldChecksum]  = GetMD5(filePath)

    if options.encrypted:
        metadataContent[FieldEncrypted] = True

    # Encode the dictionary into JSON
    metadataStr = json.dumps(metadataContent, sort_keys=True, indent=4, separators=(',', ': '))
    metadataStr = ''.join([metadataStr, '\n'])

    # Print the final content of the metadata file
    LogVerbose(metadataStr)

    # Get the path where the medatata file is written
    metadataPath = ''.join([filePath, ExtensionMetadata])
    LogVerbose(' => ' + metadataPath)

    WriteFileContent(metadataPath, metadataStr)
    if options.md5MetadataFile:
        AppendMD5(metadataPath)
    LogStd('Created %s' % (metadataPath))


#-------------------------------------------------------------------------------
# Main.
#
#-------------------------------------------------------------------------------
def main():
    global LogVerboseEnabled

    (options, args) = ParseArgs()

    # Adjust verbosity
    LogVerboseEnabled = options.verbose

    # Adjust md5 args
    if options.md5All:
        options.md5ContentFile = True
        options.md5MetadataFile = True

    # Get the root folder
    rootFolder = ''
    if os.path.isabs('options.rootFolder'):
        rootFolder = options.rootFolder
    else:
        rootFolder = os.path.join(os.getcwd(), options.rootFolder)

    LogStd('Listing in %s\n' % (rootFolder))

    for root, subFolders, files in os.walk(rootFolder):
        for file in files:
            curFilePath = os.path.join(root, file)

            if ShouldCreateMetadata(curFilePath):
                CreateMetadata(curFilePath, options)


if __name__ == "__main__":
    main()